@@ -136,7 +136,7 @@ foo_method_call (GDBusConnection       *connection,
     {
       const gchar *input;
       gchar *output;
-      g_variant_get (parameters, "(&s)", &input);
+      g_variant_get (parameters, "(s)", &input);
       output = g_strdup_printf ("You passed the string `%s'. Jolly good!", input);
       g_dbus_method_invocation_return_value (invocation, g_variant_new ("(s)", output));
       g_free (output);
@@ -314,6 +314,7 @@ introspect_callback (GDBusProxy   *proxy,
                      GAsyncResult *res,
                      gpointer      user_data)
 {
+  const gchar *s;
   gchar **xml_data = user_data;
   GVariant *result;
   GError *error;
@@ -324,7 +325,8 @@ introspect_callback (GDBusProxy   *proxy,
                                               &error);
   g_assert_no_error (error);
   g_assert (result != NULL);
-  g_variant_get (result, "(s)", xml_data);
+  g_variant_get (result, "(s)", &s);
+  *xml_data = g_strdup (s);
   g_variant_unref (result);
 
   g_main_loop_quit (loop);
@@ -343,6 +345,7 @@ get_nodes_at (GDBusConnection  *c,
 
   error = NULL;
   proxy = g_dbus_proxy_new_sync (c,
+                                 G_TYPE_DBUS_PROXY,
                                  G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
                                  G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
                                  NULL,
@@ -399,6 +402,7 @@ has_interface (GDBusConnection *c,
 
   error = NULL;
   proxy = g_dbus_proxy_new_sync (c,
+                                 G_TYPE_DBUS_PROXY,
                                  G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
                                  G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
                                  NULL,
@@ -448,6 +452,7 @@ count_interfaces (GDBusConnection *c,
 
   error = NULL;
   proxy = g_dbus_proxy_new_sync (c,
+                                 G_TYPE_DBUS_PROXY,
                                  G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
                                  G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
                                  NULL,
@@ -519,6 +524,7 @@ dyna_create (GDBusConnection *c,
 
   error = NULL;
   proxy = g_dbus_proxy_new_sync (c,
+                                 G_TYPE_DBUS_PROXY,
                                  G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
                                  G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
                                  NULL,
@@ -614,38 +620,37 @@ subtree_enumerate (GDBusConnection       *connection,
 }
 
 /* Only allows certain objects, and aborts on unknowns */
-static GDBusInterfaceInfo **
+static GPtrArray *
 subtree_introspect (GDBusConnection       *connection,
                     const gchar           *sender,
                     const gchar           *object_path,
                     const gchar           *node,
                     gpointer               user_data)
 {
-  const GDBusInterfaceInfo *interfaces[2] = {
-     NULL /* filled in below */, NULL
-  };
+  GPtrArray *interfaces;
 
   /* VPs implement the Foo interface, EVPs implement the Bar interface. The root
    * does not implement any interfaces
    */
-  if (node == NULL)
+  interfaces = g_ptr_array_new ();
+  if (g_str_has_prefix (node, "vp"))
     {
-      return NULL;
-    }
-  else if (g_str_has_prefix (node, "vp"))
-    {
-      interfaces[0] = &foo_interface_info;
+      g_ptr_array_add (interfaces, (gpointer) &foo_interface_info);
     }
   else if (g_str_has_prefix (node, "evp"))
     {
-      interfaces[0] = &bar_interface_info;
+      g_ptr_array_add (interfaces, (gpointer) &bar_interface_info);
+    }
+  else if (g_strcmp0 (node, "/") == 0)
+    {
+      /* do nothing */
     }
   else
     {
       g_assert_not_reached ();
     }
 
-  return g_memdup (interfaces, 2 * sizeof (void *));
+  return interfaces;
 }
 
 static const GDBusInterfaceVTable *
@@ -692,16 +697,20 @@ dynamic_subtree_enumerate (GDBusConnection       *connection,
 }
 
 /* Allow all objects to be introspected */
-static GDBusInterfaceInfo **
+static GPtrArray *
 dynamic_subtree_introspect (GDBusConnection       *connection,
                             const gchar           *sender,
                             const gchar           *object_path,
                             const gchar           *node,
                             gpointer               user_data)
 {
-  const GDBusInterfaceInfo *interfaces[2] = { &dyna_interface_info, NULL };
+  GPtrArray *interfaces;
 
-  return g_memdup (interfaces, 2 * sizeof (void *));
+  /* All nodes (including the root node) implements the Dyna interface */
+  interfaces = g_ptr_array_new ();
+  g_ptr_array_add (interfaces, (gpointer) &dyna_interface_info);
+
+  return interfaces;
 }
 
 static const GDBusInterfaceVTable *
@@ -738,6 +747,7 @@ test_dispatch_thread_func (gpointer user_data)
   const gchar *value_str;
 
   foo_proxy = g_dbus_proxy_new_sync (c,
+                                     G_TYPE_DBUS_PROXY,
                                      G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS |
                                      G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
                                      NULL,
@@ -773,7 +783,7 @@ test_dispatch_thread_func (gpointer user_data)
   g_assert_no_error (error);
   g_assert (value != NULL);
   g_assert (g_variant_is_of_type (value, G_VARIANT_TYPE ("(s)")));
-  g_variant_get (value, "(&s)", &value_str);
+  g_variant_get (value, "(s)", &value_str);
   g_assert_cmpstr (value_str, ==, "You passed the string `winwinwin'. Jolly good!");
   g_variant_unref (value);
 
@@ -799,7 +809,7 @@ test_dispatch_thread_func (gpointer user_data)
                                   NULL,
                                   &error);
   g_assert_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS);
-  g_assert_cmpstr (error->message, ==, "GDBus.Error:org.freedesktop.DBus.Error.InvalidArgs: Type of message, `(s)', does not match expected type `()'");
+  g_assert_cmpstr (error->message, ==, "GDBus.Error:org.freedesktop.DBus.Error.InvalidArgs: Signature of message, `s', does not match expected signature `'");
   g_error_free (error);
   g_assert (value == NULL);
 
@@ -816,18 +826,6 @@ test_dispatch_thread_func (gpointer user_data)
   g_error_free (error);
   g_assert (value == NULL);
 
-  error = NULL;
-  value = g_dbus_proxy_call_sync (foo_proxy,
-                                  "org.example.FooXYZ.NonExistant",
-                                  NULL,
-                                  G_DBUS_CALL_FLAGS_NONE,
-                                  -1,
-                                  NULL,
-                                  &error);
-  g_assert_error (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD);
-  g_error_free (error);
-  g_assert (value == NULL);
-
   /* user properties */
   error = NULL;
   value = g_dbus_proxy_call_sync (foo_proxy,
@@ -960,7 +958,6 @@ test_object_registration (void)
   guint boss_foo_reg_id;
   guint boss_bar_reg_id;
   guint worker1_foo_reg_id;
-  guint worker1p1_foo_reg_id;
   guint worker2_bar_reg_id;
   guint intern1_foo_reg_id;
   guint intern2_bar_reg_id;
@@ -986,7 +983,7 @@ test_object_registration (void)
 
   registration_id = g_dbus_connection_register_object (c,
                                                        "/foo/boss",
-                                                       (GDBusInterfaceInfo *) &foo_interface_info,
+                                                       &foo_interface_info,
                                                        &foo_vtable,
                                                        &data,
                                                        on_object_unregistered,
@@ -998,7 +995,7 @@ test_object_registration (void)
 
   registration_id = g_dbus_connection_register_object (c,
                                                        "/foo/boss",
-                                                       (GDBusInterfaceInfo *) &bar_interface_info,
+                                                       &bar_interface_info,
                                                        NULL,
                                                        &data,
                                                        on_object_unregistered,
@@ -1010,7 +1007,7 @@ test_object_registration (void)
 
   registration_id = g_dbus_connection_register_object (c,
                                                        "/foo/boss/worker1",
-                                                       (GDBusInterfaceInfo *) &foo_interface_info,
+                                                       &foo_interface_info,
                                                        NULL,
                                                        &data,
                                                        on_object_unregistered,
@@ -1020,21 +1017,9 @@ test_object_registration (void)
   worker1_foo_reg_id = registration_id;
   num_successful_registrations++;
 
-  registration_id = g_dbus_connection_register_object (c,
-                                                       "/foo/boss/worker1p1",
-                                                       (GDBusInterfaceInfo *) &foo_interface_info,
-                                                       NULL,
-                                                       &data,
-                                                       on_object_unregistered,
-                                                       &error);
-  g_assert_no_error (error);
-  g_assert (registration_id > 0);
-  worker1p1_foo_reg_id = registration_id;
-  num_successful_registrations++;
-
   registration_id = g_dbus_connection_register_object (c,
                                                        "/foo/boss/worker2",
-                                                       (GDBusInterfaceInfo *) &bar_interface_info,
+                                                       &bar_interface_info,
                                                        NULL,
                                                        &data,
                                                        on_object_unregistered,
@@ -1046,7 +1031,7 @@ test_object_registration (void)
 
   registration_id = g_dbus_connection_register_object (c,
                                                        "/foo/boss/interns/intern1",
-                                                       (GDBusInterfaceInfo *) &foo_interface_info,
+                                                       &foo_interface_info,
                                                        NULL,
                                                        &data,
                                                        on_object_unregistered,
@@ -1059,7 +1044,7 @@ test_object_registration (void)
   /* ... and try again at another path */
   registration_id = g_dbus_connection_register_object (c,
                                                        "/foo/boss/interns/intern2",
-                                                       (GDBusInterfaceInfo *) &bar_interface_info,
+                                                       &bar_interface_info,
                                                        NULL,
                                                        &data,
                                                        on_object_unregistered,
@@ -1072,7 +1057,7 @@ test_object_registration (void)
   /* register at the same path/interface - this should fail */
   registration_id = g_dbus_connection_register_object (c,
                                                        "/foo/boss/interns/intern2",
-                                                       (GDBusInterfaceInfo *) &bar_interface_info,
+                                                       &bar_interface_info,
                                                        NULL,
                                                        &data,
                                                        on_object_unregistered,
@@ -1086,7 +1071,7 @@ test_object_registration (void)
   /* register at different interface - shouldn't fail */
   registration_id = g_dbus_connection_register_object (c,
                                                        "/foo/boss/interns/intern2",
-                                                       (GDBusInterfaceInfo *) &foo_interface_info,
+                                                       &foo_interface_info,
                                                        NULL,
                                                        &data,
                                                        on_object_unregistered,
@@ -1104,7 +1089,7 @@ test_object_registration (void)
   /* register it back */
   registration_id = g_dbus_connection_register_object (c,
                                                        "/foo/boss/interns/intern2",
-                                                       (GDBusInterfaceInfo *) &foo_interface_info,
+                                                       &foo_interface_info,
                                                        NULL,
                                                        &data,
                                                        on_object_unregistered,
@@ -1116,7 +1101,7 @@ test_object_registration (void)
 
   registration_id = g_dbus_connection_register_object (c,
                                                        "/foo/boss/interns/intern3",
-                                                       (GDBusInterfaceInfo *) &bar_interface_info,
+                                                       &bar_interface_info,
                                                        NULL,
                                                        &data,
                                                        on_object_unregistered,
@@ -1172,7 +1157,7 @@ test_object_registration (void)
    */
   registration_id = g_dbus_connection_register_object (c,
                                                        "/foo/boss/executives/non_subtree_object",
-                                                       (GDBusInterfaceInfo *) &bar_interface_info,
+                                                       &bar_interface_info,
                                                        NULL,
                                                        &data,
                                                        on_object_unregistered,
@@ -1183,7 +1168,7 @@ test_object_registration (void)
   num_successful_registrations++;
   registration_id = g_dbus_connection_register_object (c,
                                                        "/foo/boss/executives/non_subtree_object",
-                                                       (GDBusInterfaceInfo *) &foo_interface_info,
+                                                       &foo_interface_info,
                                                        NULL,
                                                        &data,
                                                        on_object_unregistered,
@@ -1260,9 +1245,8 @@ test_object_registration (void)
 
   nodes = get_nodes_at (c, "/foo/boss");
   g_assert (nodes != NULL);
-  g_assert_cmpint (g_strv_length (nodes), ==, 5);
+  g_assert_cmpint (g_strv_length (nodes), ==, 4);
   g_assert (_g_strv_has_string ((const gchar* const *) nodes, "worker1"));
-  g_assert (_g_strv_has_string ((const gchar* const *) nodes, "worker1p1"));
   g_assert (_g_strv_has_string ((const gchar* const *) nodes, "worker2"));
   g_assert (_g_strv_has_string ((const gchar* const *) nodes, "interns"));
   g_assert (_g_strv_has_string ((const gchar* const *) nodes, "executives"));
@@ -1322,17 +1306,6 @@ test_object_registration (void)
   g_assert (_g_strv_has_string ((const gchar* const *) nodes, "evp2"));
   g_strfreev (nodes);
 
-  /* This is to check that a bug (rather, class of bugs) in gdbusconnection.c's
-   *
-   *  g_dbus_connection_list_registered_unlocked()
-   *
-   * where /foo/boss/worker1 reported a child '1', is now fixed.
-   */
-  nodes = get_nodes_at (c, "/foo/boss/worker1");
-  g_assert (nodes != NULL);
-  g_assert_cmpint (g_strv_length (nodes), ==, 0);
-  g_strfreev (nodes);
-
   /* check that calls are properly dispatched to the functions in foo_vtable for objects
    * implementing the org.example.Foo interface
    *
@@ -1361,7 +1334,6 @@ test_object_registration (void)
   g_assert (g_dbus_connection_unregister_object (c, boss_foo_reg_id));
   g_assert (g_dbus_connection_unregister_object (c, boss_bar_reg_id));
   g_assert (g_dbus_connection_unregister_object (c, worker1_foo_reg_id));
-  g_assert (g_dbus_connection_unregister_object (c, worker1p1_foo_reg_id));
   g_assert (g_dbus_connection_unregister_object (c, worker2_bar_reg_id));
   g_assert (g_dbus_connection_unregister_object (c, intern1_foo_reg_id));
   g_assert (g_dbus_connection_unregister_object (c, intern2_bar_reg_id));
